@@ -18,7 +18,6 @@
 # MA 02110-1301 USA.
 #
 
-import os
 import logging
 import traceback
 
@@ -28,7 +27,8 @@ import gtk.gdk
 import gtk.glade
 
 import virtinst
-from virtinst import VirtualCharDevice, VirtualDevice, VirtualVideoDevice
+from virtinst import (VirtualCharDevice, VirtualDevice, VirtualVideoDevice,
+                      VirtualWatchdog)
 
 import virtManager.util as util
 import virtManager.uihelpers as uihelpers
@@ -51,7 +51,8 @@ PAGE_SOUND = 5
 PAGE_HOSTDEV = 6
 PAGE_CHAR = 7
 PAGE_VIDEO = 8
-PAGE_SUMMARY = 9
+PAGE_WATCHDOG = 9
+PAGE_SUMMARY = 10
 
 char_widget_mappings = {
     "source_path" : "char-path",
@@ -63,18 +64,6 @@ char_widget_mappings = {
     "protocol" : "char-use-telnet",
 }
 
-def build_video_combo(vm, video_dev):
-    video_dev_model = gtk.ListStore(str)
-    video_dev.set_model(video_dev_model)
-    text = gtk.CellRendererText()
-    video_dev.pack_start(text, True)
-    video_dev.add_attribute(text, 'text', 0)
-    video_dev_model.set_sort_column_id(0, gtk.SORT_ASCENDING)
-    for m in VirtualVideoDevice(vm.get_connection().vmm).model_types:
-        video_dev_model.append([m])
-    if len(video_dev_model) > 0:
-        video_dev.set_active(0)
-
 class vmmAddHardware(gobject.GObject):
     __gsignals__ = {
         "action-show-help": (gobject.SIGNAL_RUN_FIRST,
@@ -94,6 +83,9 @@ class vmmAddHardware(gobject.GObject):
 
         self.storage_browser = None
 
+        # Host space polling
+        self.host_storage_timer = None
+
         self._dev = None
 
         self.topwin.hide()
@@ -108,10 +100,8 @@ class vmmAddHardware(gobject.GObject):
 
             "on_hardware_type_changed"  : self.hardware_type_changed,
 
-            "on_storage_partition_address_browse_clicked" : self.browse_storage_partition_address,
-            "on_storage_file_address_browse_clicked" : self.browse_storage_file_address,
-            "on_storage_file_address_changed": self.toggle_storage_size,
-            "on_storage_toggled" : self.change_storage_type,
+            "on_config_storage_browse_clicked": self.browse_storage,
+            "on_config_storage_select_toggled": self.toggle_storage_select,
 
             "on_mac_address_clicked" : self.change_macaddr_use,
 
@@ -166,13 +156,21 @@ class vmmAddHardware(gobject.GObject):
 
     def show(self):
         self.reset_state()
-        self.topwin.show()
         self.topwin.present()
 
     def close(self, ignore1=None,ignore2=None):
         self.topwin.hide()
+        self.remove_timers()
         return 1
 
+    def remove_timers(self):
+        try:
+            if self.host_storage_timer:
+                gobject.source_remove(self.host_storage_timer)
+                self.host_storage_timer = None
+        except:
+            pass
+
     def is_visible(self):
         if self.topwin.flags() & gtk.VISIBLE:
             return 1
@@ -209,18 +207,15 @@ class vmmAddHardware(gobject.GObject):
 
         # Virtual network list
         net_list = self.window.get_widget("net-list")
-        uihelpers.init_network_list(net_list)
+        bridge_box = self.window.get_widget("net-bridge-box")
+        uihelpers.init_network_list(net_list, bridge_box)
 
         # Network model list
         netmodel_list  = self.window.get_widget("net-model")
-        netmodel_model = gtk.ListStore(str, str)
-        netmodel_list.set_model(netmodel_model)
-        text = gtk.CellRendererText()
-        netmodel_list.pack_start(text, True)
-        netmodel_list.add_attribute(text, 'text', 1)
+        uihelpers.build_netmodel_combo(self.vm, netmodel_list)
 
         # Disk device type / bus
-        target_list = self.window.get_widget("target-device")
+        target_list = self.window.get_widget("config-storage-devtype")
         target_model = gtk.ListStore(str, str, str, str, int)
         target_list.set_model(target_model)
         icon = gtk.CellRendererPixbuf()
@@ -232,6 +227,10 @@ class vmmAddHardware(gobject.GObject):
         target_list.pack_start(text, True)
         target_list.add_attribute(text, 'text', 3)
 
+        # Sparse tooltip
+        sparse_info = self.window.get_widget("config-storage-nosparse-info")
+        uihelpers.set_sparse_tooltip(sparse_info)
+
         # Input device type
         input_list = self.window.get_widget("input-type")
         input_model = gtk.ListStore(str, str, str, bool)
@@ -251,11 +250,7 @@ class vmmAddHardware(gobject.GObject):
 
         # Sound model list
         sound_list = self.window.get_widget("sound-model")
-        sound_lmodel = gtk.ListStore(str)
-        sound_list.set_model(sound_lmodel)
-        text = gtk.CellRendererText()
-        sound_list.pack_start(text, True)
-        sound_list.add_attribute(text, 'text', 0)
+        uihelpers.build_sound_combo(self.vm, sound_list)
 
         host_devtype = self.window.get_widget("host-device-type")
         # Description, nodedev type, specific type capability, sub type,
@@ -277,7 +272,7 @@ class vmmAddHardware(gobject.GObject):
 
         # Video device
         video_dev = self.window.get_widget("video-model")
-        build_video_combo(self.vm, video_dev)
+        uihelpers.build_video_combo(self.vm, video_dev)
 
         char_devtype = self.window.get_widget("char-device-type")
         # Type name, desc
@@ -305,7 +300,18 @@ class vmmAddHardware(gobject.GObject):
 
         self.window.get_widget("char-info-box").modify_bg(gtk.STATE_NORMAL, gtk.gdk.color_parse("grey"))
 
+        # Watchdog widgets
+        combo = self.window.get_widget("watchdog-model")
+        uihelpers.build_watchdogmodel_combo(self.vm, combo)
+
+        combo = self.window.get_widget("watchdog-action")
+        uihelpers.build_watchdogaction_combo(self.vm, combo)
+
+
     def reset_state(self):
+        is_local = not self.conn.is_remote()
+        is_storage_capable = self.conn.is_storage_capable()
+
         notebook = self.window.get_widget("create-pages")
         notebook.set_current_page(0)
 
@@ -314,24 +320,30 @@ class vmmAddHardware(gobject.GObject):
         self.window.get_widget("create-forward").show()
         self.window.get_widget("create-forward").grab_focus()
         self.window.get_widget("create-back").set_sensitive(False)
-        self.window.get_widget("storage-file-size").set_sensitive(False)
         self.window.get_widget("create-help").hide()
 
         # Storage init
-        self.change_storage_type()
-        if os.getuid() == 0:
-            self.window.get_widget("storage-partition").set_active(True)
-        else:
-            self.window.get_widget("storage-file-backed").set_active(True)
-        self.window.get_widget("storage-partition-address").set_text("")
-        self.window.get_widget("storage-file-address").set_text("")
-        self.window.get_widget("storage-file-size").set_value(4000)
-        self.window.get_widget("non-sparse").set_active(True)
-        target_list = self.window.get_widget("target-device")
+        label_widget = self.window.get_widget("phys-hd-label")
+        if not self.host_storage_timer:
+            self.host_storage_timer = util.safe_timeout_add(3 * 1000,
+                                                uihelpers.host_space_tick,
+                                                self.conn, self.config,
+                                                label_widget)
+        self.window.get_widget("config-storage-create").set_active(True)
+        self.window.get_widget("config-storage-size").set_value(8)
+        self.window.get_widget("config-storage-entry").set_text("")
+        self.window.get_widget("config-storage-nosparse").set_active(True)
+        target_list = self.window.get_widget("config-storage-devtype")
         self.populate_target_device_model(target_list.get_model())
         if len(target_list.get_model()) > 0:
             target_list.set_active(0)
 
+        have_storage = (is_local or is_storage_capable)
+        storage_tooltip = None
+        if not have_storage:
+            storage_tooltip = _("Connection does not support storage"
+                                " management.")
+
         # Network init
         newmac = uihelpers.generate_macaddr(self.vm.get_connection())
         self.window.get_widget("mac-address").set_active(bool(newmac))
@@ -350,9 +362,8 @@ class vmmAddHardware(gobject.GObject):
             net_warn.hide()
 
         netmodel = self.window.get_widget("net-model")
-        self.populate_network_model_model(netmodel.get_model())
-        if len(netmodel.get_model()) > 0:
-            netmodel.set_active(0)
+        uihelpers.populate_netmodel_combo(self.vm, netmodel)
+        netmodel.set_active(0)
 
         # Input device init
         input_box = self.window.get_widget("input-type")
@@ -372,9 +383,8 @@ class vmmAddHardware(gobject.GObject):
 
         # Sound init
         sound_box = self.window.get_widget("sound-model")
-        self.populate_sound_model_model(sound_box.get_model())
         sound_box.set_active(0)
-
+                
         # Hostdev init
         host_devtype = self.window.get_widget("host-device-type")
         self.populate_host_device_type_model(host_devtype.get_model())
@@ -398,7 +408,8 @@ class vmmAddHardware(gobject.GObject):
         def add_hw_option(name, icon, page, sensitive, tooltip):
             model.append([name, icon, page, sensitive, tooltip])
 
-        add_hw_option("Storage", "drive-harddisk", PAGE_DISK, True, None)
+        add_hw_option("Storage", "drive-harddisk", PAGE_DISK, have_storage,
+                      have_storage and storage_tooltip or None)
         add_hw_option("Network", "network-idle", PAGE_NETWORK, True, None)
         add_hw_option("Input", "input-mouse", PAGE_INPUT, self.vm.is_hvm(),
                       _("Not supported for this guest type."))
@@ -422,6 +433,9 @@ class vmmAddHardware(gobject.GObject):
                             self.vm.get_connection().vmm,
                             virtinst.support.SUPPORT_CONN_DOMAIN_VIDEO),
                       _("Libvirt version does not support video devices."))
+        add_hw_option("Watchdog", "device_pci", PAGE_WATCHDOG,
+                      self.vm.is_hvm(),
+                      _("Not supported for this guest type."))
 
         self.window.get_widget("hardware-type").set_active(0)
 
@@ -429,21 +443,6 @@ class vmmAddHardware(gobject.GObject):
     # UI population methods #
     #########################
 
-    def populate_network_model_model(self, model):
-        model.clear()
-
-        # [xml value, label]
-        model.append([None, _("Hypervisor default")])
-        if self.vm.is_hvm():
-            mod_list = [ "rtl8139", "ne2k_pci", "pcnet" ]
-            if self.vm.get_hv_type() == "kvm":
-                mod_list.append("e1000")
-                mod_list.append("virtio")
-            mod_list.sort()
-
-            for m in mod_list:
-                model.append([m, m])
-
     def populate_target_device_model(self, model):
         model.clear()
         #[bus, device, icon, desc, iconsize]
@@ -480,13 +479,6 @@ class vmmAddHardware(gobject.GObject):
         model.append([_("VNC server"), "vnc"])
         model.append([_("Local SDL window"), "sdl"])
 
-    def populate_sound_model_model(self, model):
-        model.clear()
-        lst = virtinst.VirtualAudio.MODELS
-        lst.sort()
-        for m in lst:
-            model.append([m])
-
     def populate_host_device_type_model(self, model):
         model.clear()
         for m in [ ["PCI Device", "pci", None, "net", "80203"],
@@ -526,46 +518,33 @@ class vmmAddHardware(gobject.GObject):
         return _type.get_model().get_value(_type.get_active_iter(), 2)
 
     # Disk getters
-    def get_config_disk_image(self):
-        if self.window.get_widget("storage-partition").get_active():
-            return self.window.get_widget("storage-partition-address").get_text()
-        else:
-            return self.window.get_widget("storage-file-address").get_text()
+    def is_default_storage(self):
+        return self.window.get_widget("config-storage-create").get_active()
 
-    def get_config_partition_size(self):
-        try:
-            partition_address = self.get_config_disk_image()
-            fd = open(partition_address,"rb")
-            fd.seek(0,2)
-            block_size = fd.tell() / 1024 / 1024
-            return block_size
-        except Exception:
-            details = "Unable to verify partition size: '%s'" % \
-                      "".join(traceback.format_exc())
-            logging.error(details)
-            return None
+    def get_storage_info(self):
+        path = None
+        size = self.window.get_widget("config-storage-size").get_value()
+        sparse = not self.window.get_widget("config-storage-nosparse").get_active()
 
-    def get_config_disk_size(self):
-        if self.window.get_widget("storage-partition").get_active():
-            return self.get_config_partition_size()
-        if not self.window.get_widget("storage-file-backed").get_active():
-            return None
-        if not self.window.get_widget("storage-file-size").get_editable():
-            return None
+        if self.is_default_storage():
+            path = util.get_default_path(self.conn, self.config,
+                                         self.vm.get_name())
+            logging.debug("Default storage path is: %s" % path)
         else:
-            return self.window.get_widget("storage-file-size").get_value()
+            path = self.window.get_widget("config-storage-entry").get_text()
+
+        return (path, size, sparse)
 
     def get_config_disk_target(self):
-        target = self.window.get_widget("target-device")
-        bus = target.get_model().get_value(target.get_active_iter(), 0)
-        device = target.get_model().get_value(target.get_active_iter(), 1)
-        return bus, device
+        target = self.window.get_widget("config-storage-devtype")
+        model = target.get_model()
+        idx = target.get_active()
+        if idx == -1:
+            return None, None
 
-    def is_sparse_file(self):
-        if self.window.get_widget("non-sparse").get_active():
-            return False
-        else:
-            return True
+        bus = model[idx][0]
+        device = model[idx][1]
+        return bus, device
 
     # Input getters
     def get_config_input(self):
@@ -609,17 +588,12 @@ class vmmAddHardware(gobject.GObject):
     # Network getters
     def get_config_network(self):
         net_list = self.window.get_widget("net-list")
-        selection = net_list.get_active()
-        model = net_list.get_model()
+        bridge_ent = self.window.get_widget("net-bridge")
 
-        nettype = None
-        devname = None
-        if selection >= 0:
-            row = model[selection]
-            nettype = row[0]
-            devname = row[1]
+        net_type, net_src = uihelpers.get_network_selection(net_list,
+                                                            bridge_ent)
 
-        return (nettype, devname)
+        return net_type, net_src
 
     def get_config_net_model(self):
         model = self.window.get_widget("net-model")
@@ -656,6 +630,13 @@ class vmmAddHardware(gobject.GObject):
         modbox = self.window.get_widget("video-model")
         return modbox.get_model()[modbox.get_active()][0]
 
+    # Watchdog getters
+    def get_config_watchdog_model(self):
+        modbox = self.window.get_widget("watchdog-model")
+        return modbox.get_model()[modbox.get_active()][0]
+    def get_config_watchdog_action(self):
+        modbox = self.window.get_widget("watchdog-action")
+        return modbox.get_model()[modbox.get_active()][0]
 
     ################
     # UI listeners #
@@ -710,6 +691,9 @@ class vmmAddHardware(gobject.GObject):
         self.window.get_widget("create-forward").show()
 
     def page_changed(self, notebook, page, page_number):
+        devbox = self.window.get_widget("host-device")
+        devbox.hide()
+
         if page_number == PAGE_CHAR:
             devtype = self.window.get_widget("char-device-type")
             self.change_char_device_type(devtype)
@@ -718,6 +702,9 @@ class vmmAddHardware(gobject.GObject):
         elif page_number == PAGE_SUMMARY:
             self.populate_summary()
 
+        elif page_number == PAGE_HOSTDEV:
+            devbox.show()
+
         return
 
     def populate_summary(self):
@@ -747,14 +734,14 @@ class vmmAddHardware(gobject.GObject):
             summary_table.show_all()
 
         if hwpage == PAGE_DISK:
-            size = self.get_config_disk_size()
-            bus, target = self.get_config_disk_target()
+            size_str = (self._dev.size and ("%2.2f GB" % self._dev.size)
+                                       or "-")
 
             info_list = [
-                (_("Disk image:"),  self.get_config_disk_image()),
-                (_("Disk size:"),   size != None and "%s MB" % size or "-"),
-                (_("Device type:"), target),
-                (_("Bus type:"),    bus),
+                (_("Disk image:"),  self._dev.path),
+                (_("Disk size:"),   size_str),
+                (_("Device type:"), self._dev.device),
+                (_("Bus type:"),    self._dev.bus),
             ]
             title = _("Storage")
 
@@ -863,29 +850,25 @@ class vmmAddHardware(gobject.GObject):
             ]
             title = _("Video")
 
+        elif hwpage == PAGE_WATCHDOG:
+            title = _("Watchdog")
+            info_list = [
+                (_("Model:"), self._dev.model),
+                (_("Action:"), self._dev.get_action_desc(self._dev.action))
+            ]
+
         set_table(title, info_list)
 
 
     def finish(self, ignore=None):
-        hw = self.get_config_hardware_type()
-
         self.topwin.set_sensitive(False)
         self.topwin.window.set_cursor(gtk.gdk.Cursor(gtk.gdk.WATCH))
 
-        func_dict = { PAGE_NETWORK: self.add_network,
-                      PAGE_DISK: self.add_storage,
-                      PAGE_INPUT: self.add_input,
-                      PAGE_GRAPHICS: self.add_graphics,
-                      PAGE_SOUND: self.add_sound,
-                      PAGE_HOSTDEV: self.add_hostdev,
-                      PAGE_CHAR: self.add_device,
-                      PAGE_VIDEO: self.add_device}
-
         try:
-            func = func_dict[hw]
-            errinfo = func()
+            failure, errinfo = self.add_device()
             error, details = errinfo or (None, None)
         except Exception, e:
+            failure = True
             error = _("Unable to add device: %s") % str(e)
             details = "".join(traceback.format_exc())
 
@@ -895,50 +878,19 @@ class vmmAddHardware(gobject.GObject):
         self.topwin.set_sensitive(True)
         self.topwin.window.set_cursor(gtk.gdk.Cursor(gtk.gdk.TOP_LEFT_ARROW))
 
-        if not error:
+        if not failure:
             self.close()
 
     # Storage listeners
-    def browse_storage_partition_address(self, src, ignore=None):
-        textent = self.window.get_widget("storage-partition-address")
+    def browse_storage(self, ignore1):
+        self._browse_file(self.window.get_widget("config-storage-entry"))
 
-        self._browse_file(textent)
+    def toggle_storage_select(self, src):
+        act = src.get_active()
+        self.window.get_widget("config-storage-browse-box").set_sensitive(act)
 
-    def browse_storage_file_address(self, src, ignore=None):
-        textent = self.window.get_widget("storage-file-address")
-
-        self._browse_file(textent, confirm_overwrite=True)
-
-    def toggle_storage_size(self, ignore1=None, ignore2=None):
-        filename = self.get_config_disk_image()
-        if filename != None and len(filename) > 0 and \
-           (self.vm.get_connection().is_remote() or
-            not os.path.exists(filename)):
-            self.window.get_widget("storage-file-size").set_sensitive(True)
-            self.window.get_widget("non-sparse").set_sensitive(True)
-            size = self.get_config_disk_size()
-            if size == None:
-                size = 4000
-            self.window.get_widget("storage-file-size").set_value(size)
-        else:
-            self.window.get_widget("storage-file-size").set_sensitive(False)
-            self.window.get_widget("non-sparse").set_sensitive(False)
-            if os.path.isfile(filename):
-                size = os.path.getsize(filename)/(1024*1024)
-                self.window.get_widget("storage-file-size").set_value(size)
-            else:
-                self.window.get_widget("storage-file-size").set_value(0)
-
-    def change_storage_type(self, ignore=None):
-        if self.window.get_widget("storage-partition").get_active():
-            self.window.get_widget("storage-partition-box").set_sensitive(True)
-            self.window.get_widget("storage-file-box").set_sensitive(False)
-            self.window.get_widget("storage-file-size").set_sensitive(False)
-            self.window.get_widget("non-sparse").set_sensitive(False)
-        else:
-            self.window.get_widget("storage-partition-box").set_sensitive(False)
-            self.window.get_widget("storage-file-box").set_sensitive(True)
-            self.toggle_storage_size()
+    def set_disk_storage_path(self, ignore, path):
+        self.window.get_widget("config-storage-entry").set_text(path)
 
     # Network listeners
     def change_macaddr_use(self, ignore=None):
@@ -1031,56 +983,36 @@ class vmmAddHardware(gobject.GObject):
     # Add device methods #
     ######################
 
-    def add_network(self):
-        self._dev.setup(self.vm.get_connection().vmm)
-        self.add_device(self._dev.get_xml_config())
+    def setup_device(self):
+        if (self._dev.virtual_device_type ==
+            virtinst.VirtualDevice.VIRTUAL_DEV_DISK):
+            progWin = vmmAsyncJob(self.config, self.do_file_allocate,
+                                  [self._dev],
+                                  title=_("Creating Storage File"),
+                                  text=_("Allocation of disk storage may take "
+                                         "a few minutes to complete."))
+            progWin.run()
 
-    def add_input(self):
-        inp = self.get_config_input()
-        xml = "<input type='%s' bus='%s'/>\n" % (inp[1], inp[2])
-        self.add_device(xml)
+            error, details = progWin.get_error()
+            if error:
+                return (error, details)
 
-    def add_graphics(self):
-        self.add_device(self._dev.get_xml_config())
-
-    def add_sound(self):
-        self.add_device(self._dev.get_xml_config())
-
-    def add_hostdev(self):
-        self._dev.setup()
-        self.add_device(self._dev.get_xml_config())
-
-    def add_storage(self):
-        used = []
-        disks = (self.vm.get_disk_devices() +
-                 self.vm.get_disk_devices(inactive=True))
-        for d in disks:
-            used.append(d[2])
-
-        self._dev.generate_target(used)
-
-        progWin = vmmAsyncJob(self.config, self.do_file_allocate, [self._dev],
-                              title=_("Creating Storage File"),
-                              text=_("Allocation of disk storage may take "
-                                     "a few minutes to complete."))
-        progWin.run()
-
-        error, details = progWin.get_error()
-        if error == None:
-            self.add_device(self._dev.get_xml_config())
         else:
-            return (error, details)
+            self._dev.setup_dev(self.conn.vmm)
 
-    def add_device(self, xml=None):
-        if not xml:
-            xml = self._dev.get_xml_config()
+    def add_device(self):
+        ret = self.setup_device()
+        if ret:
+            # Encountered an error
+            return (True, ret)
 
-        logging.debug("Adding device:\n" + xml)
+        self._dev.get_xml_config()
+        logging.debug("Adding device:\n" + self._dev.get_xml_config())
 
         # Hotplug device
         attach_err = False
         try:
-            self.vm.attach_device(xml)
+            self.vm.attach_device(self._dev)
         except Exception, e:
             logging.debug("Device could not be hotplugged: %s" % str(e))
             attach_err = True
@@ -1092,15 +1024,17 @@ class vmmAddHardware(gobject.GObject):
                                      "the running machine. Would you like to "
                                      "make the device available after the "
                                      "next VM shutdown?")):
-                return
+                return (False, None)
 
         # Alter persistent config
         try:
-            self.vm.add_device(xml)
+            self.vm.add_device(self._dev)
         except Exception, e:
             self.err.show_err(_("Error adding device: %s" % str(e)),
                               "".join(traceback.format_exc()))
-            return
+            return (True, None)
+
+        return (False, None)
 
     def do_file_allocate(self, disk, asyncjob):
         meter = vmmCreateMeter(asyncjob)
@@ -1112,7 +1046,7 @@ class vmmAddHardware(gobject.GObject):
                 newconn = util.dup_lib_conn(self.config, disk.conn)
                 disk.conn = newconn
             logging.debug("Starting background file allocate process")
-            disk.setup(meter)
+            disk.setup_dev(self.conn.vmm, meter=meter)
             logging.debug("Allocation completed")
         except Exception, e:
             details = (_("Unable to complete install: '%s'") %
@@ -1133,7 +1067,7 @@ class vmmAddHardware(gobject.GObject):
         elif page_num == PAGE_NETWORK:
             return self.validate_page_network()
         elif page_num == PAGE_INPUT:
-            return True
+            return self.validate_page_input()
         elif page_num == PAGE_GRAPHICS:
             return self.validate_page_graphics()
         elif page_num == PAGE_SOUND:
@@ -1144,6 +1078,8 @@ class vmmAddHardware(gobject.GObject):
             return self.validate_page_char()
         elif page_num == PAGE_VIDEO:
             return self.validate_page_video()
+        elif page_num == PAGE_WATCHDOG:
+            return self.validate_page_watchdog()
 
     def validate_page_intro(self):
         if self.get_config_hardware_type() == None:
@@ -1152,61 +1088,90 @@ class vmmAddHardware(gobject.GObject):
         self._dev = None
 
     def validate_page_storage(self):
-        path = self.get_config_disk_image()
-        if not path:
-            return self.err.val_err(_("Storage Path Required"),
-                _("You must specify a partition or a file for disk storage."))
-
-        if self.window.get_widget("target-device").get_active() == -1:
-            return self.err.val_err(_("Target Device Required"),
-                        _("You must select a target device for the disk."))
-
         bus, device = self.get_config_disk_target()
 
-        # Build disk object
-        filesize = self.get_config_disk_size()
-        if self.get_config_disk_size() != None:
-            filesize = self.get_config_disk_size() / 1024.0
         readonly = False
         if device == virtinst.VirtualDisk.DEVICE_CDROM:
             readonly=True
 
         try:
-            if (os.path.dirname(os.path.abspath(path)) ==
-                util.DEFAULT_POOL_PATH):
-                util.build_default_pool(self.vm.get_connection().vmm)
+            # This can error out
+            diskpath, disksize, sparse = self.get_storage_info()
 
-            self._dev = virtinst.VirtualDisk(self.get_config_disk_image(),
-                                             filesize,
-                                             sparse=self.is_sparse_file(),
-                                             readOnly=readonly,
-                                             device=device,
-                                             bus=bus,
-                                             conn=self.vm.get_connection().vmm)
+            if self.is_default_storage():
+                # See if the ideal disk path (/default/pool/vmname.img)
+                # exists, and if unused, prompt the use for using it
+                ideal = util.get_ideal_path(self.conn, self.config,
+                                            self.vm.get_name())
+                do_exist = False
+                ret = True
 
-            if (self._dev.type == virtinst.VirtualDisk.TYPE_FILE and
-                not self.vm.is_hvm() and virtinst.util.is_blktap_capable()):
-                self._dev.driver_name = virtinst.VirtualDisk.DRIVER_TAP
+                try:
+                    do_exist = virtinst.VirtualDisk.path_exists(
+                                                        self.conn.vmm, ideal)
 
-        except ValueError, e:
-            return self.err.val_err(_("Invalid Storage Parameters"), str(e))
+                    ret = virtinst.VirtualDisk.path_in_use_by(self.conn.vmm,
+                                                              ideal)
+                except:
+                    logging.exception("Error checking default path usage")
 
-        ret = self._dev.is_size_conflict()
-        if not ret[0] and ret[1]:
-            res = self.err.ok_cancel(_("Not Enough Free Space"), ret[1])
+                if do_exist and not ret:
+                    do_use = self.err.yes_no(
+                        _("The following path already exists, but is not\n"
+                          "in use by any virtual machine:\n\n%s\n\n"
+                          "Would you like to use this path?") % ideal)
+
+                    if do_use:
+                        diskpath = ideal
+
+            if not diskpath:
+                return self.err.val_err(_("A storage path must be specified."))
+
+            disk = virtinst.VirtualDisk(conn = self.conn.vmm,
+                                        path = diskpath,
+                                        size = disksize,
+                                        sparse = sparse,
+                                        readOnly = readonly,
+                                        device = device,
+                                        bus = bus)
+
+            if (disk.type == virtinst.VirtualDisk.TYPE_FILE and
+                not self.vm.is_hvm() and
+                virtinst.util.is_blktap_capable()):
+                disk.driver_name = virtinst.VirtualDisk.DRIVER_TAP
+
+        except Exception, e:
+            return self.err.val_err(_("Storage parameter error."), str(e))
+
+        # Generate target
+        used = []
+        disks = (self.vm.get_disk_devices() +
+                 self.vm.get_disk_devices(inactive=True))
+        for d in disks:
+            used.append(d[2])
+
+        disk.generate_target(used)
+
+        isfatal, errmsg = disk.is_size_conflict()
+        if not isfatal and errmsg:
+            # Fatal errors are reported when setting 'size'
+            res = self.err.ok_cancel(_("Not Enough Free Space"), errmsg)
             if not res:
                 return False
 
-        if self._dev.is_conflict_disk(self.vm.get_connection().vmm) is True:
-            res = self.err.yes_no(
-                _('Disk "%s" is already in use by another guest!' % self._dev),
-                _("Do you really want to use the disk?"))
+        # Disk collision
+        if disk.is_conflict_disk(self.conn.vmm):
+            res = self.err.yes_no(_('Disk "%s" is already in use by another '
+                                    'guest!' %  disk.path),
+                                  _("Do you really want to use the disk?"))
             if not res:
                 return False
 
         uihelpers.check_path_search_for_qemu(self.topwin, self.config,
-                                             self.conn, self._dev.path)
+                                             self.conn, disk.path)
 
+        self._dev = disk
+        return True
 
 
     def validate_page_network(self):
@@ -1229,6 +1194,14 @@ class vmmAddHardware(gobject.GObject):
 
         self._dev = ret
 
+    def validate_page_input(self):
+        ignore, inp_type, inp_bus = self.get_config_input()
+        dev = virtinst.VirtualInputDevice(self.conn.vmm)
+        dev.type = inp_type
+        dev.bus = inp_bus
+
+        self._dev = dev
+
     def validate_page_graphics(self):
         graphics = self.get_config_graphics()
         if graphics == "vnc":
@@ -1304,7 +1277,7 @@ class vmmAddHardware(gobject.GObject):
             # Dump XML for sanity checking
             self._dev.get_xml_config()
         except Exception, e:
-            return self.err.val_err(_("%s device parameter error.") %
+            return self.err.val_err(_("%s device parameter error") %
                                     chartype.capitalize(), str(e))
 
     def validate_page_video(self):
@@ -1315,7 +1288,20 @@ class vmmAddHardware(gobject.GObject):
             self._dev = VirtualVideoDevice(conn=conn)
             self._dev.model_type = model
         except Exception, e:
-            return self.err.val_err(_("Video device parameter error.") %
+            return self.err.val_err(_("Video device parameter error"),
+                                    str(e))
+
+    def validate_page_watchdog(self):
+        conn = self.vm.get_connection().vmm
+        model = self.get_config_watchdog_model()
+        action = self.get_config_watchdog_action()
+
+        try:
+            self._dev = VirtualWatchdog(conn=conn)
+            self._dev.model = model
+            self._dev.action = action
+        except Exception, e:
+            return self.err.val_err(_("Watchdog parameter error"),
                                     str(e))
 
 
@@ -1324,27 +1310,17 @@ class vmmAddHardware(gobject.GObject):
     # Unsorted helpers #
     ####################
 
-    def _browse_file(self, textent, confirm_overwrite=False):
-        def confirm_cb(chooser):
-            # Only called when the user has chosen an existing file
-            self.window.get_widget("storage-file-size").set_sensitive(False)
-            return gtk.FILE_CHOOSER_CONFIRMATION_ACCEPT_FILENAME
-
+    def _browse_file(self, textent):
         def set_storage_cb(src, path):
             if path:
                 textent.set_text(path)
 
-        confirm_func = None
-        if confirm_overwrite:
-            confirm_func = confirm_cb
-
         conn = self.vm.get_connection()
         if self.storage_browser == None:
             self.storage_browser = vmmStorageBrowser(self.config, conn)
 
         self.storage_browser.set_finish_cb(set_storage_cb)
         self.storage_browser.set_browse_reason(self.config.CONFIG_DIR_IMAGE)
-        self.storage_browser.set_local_arg("confirm_func", confirm_func)
 
         self.storage_browser.show(conn)
 
