@@ -19,30 +19,41 @@
 #
 import gtk.glade
 import gobject
-import logging
 
 import virtinst
 
-from virtManager import util
-from virtManager.opticalhelper import vmmOpticalDriveHelper
+import virtManager.uihelpers as uihelpers
+import virtManager.util as util
+from virtManager.mediadev import MEDIA_FLOPPY
+from virtManager.storagebrowse import vmmStorageBrowser
 from virtManager.error import vmmErrorDialog
 
 class vmmChooseCD(gobject.GObject):
-    __gsignals__ = {"cdrom-chosen": (gobject.SIGNAL_RUN_FIRST,
-                           gobject.TYPE_NONE,
-                           (str, str, str)), # type, source, target
-}
-    def __init__(self, config, dev_id_info, connection):
+    __gsignals__ = {
+        "cdrom-chosen": (gobject.SIGNAL_RUN_FIRST,
+                         gobject.TYPE_NONE,
+                         (str, str, str)), # type, source, target
+    }
+
+    IS_FLOPPY = 1
+    IS_CDROM  = 2
+
+    def __init__(self, config, dev_id_info, connection, media_type):
         self.__gobject_init__()
         self.window = gtk.glade.XML(config.get_glade_dir() + "/vmm-choose-cd.glade", "vmm-choose-cd", domain="virt-manager")
-        self.err = vmmErrorDialog(self.window.get_widget("vmm-choose-cd"),
+        self.topwin = self.window.get_widget("vmm-choose-cd")
+        self.topwin.hide()
+
+        self.err = vmmErrorDialog(self.topwin,
                                   0, gtk.MESSAGE_ERROR, gtk.BUTTONS_CLOSE,
                                   _("Unexpected Error"),
                                   _("An unexpected error occurred"))
+
         self.config = config
-        self.window.get_widget("vmm-choose-cd").hide()
         self.dev_id_info = dev_id_info
         self.conn = connection
+        self.storage_browser = None
+        self.media_type = media_type
 
         self.window.signal_autoconnect({
             "on_media_toggled": self.media_toggled,
@@ -55,17 +66,7 @@ class vmmChooseCD(gobject.GObject):
 
         self.window.get_widget("iso-image").set_active(True)
 
-        # set up the list for the cd-path widget
-        cd_list = self.window.get_widget("cd-path")
-        # Fields are raw device path, volume label, flag indicating
-        # whether volume is present or not, and HAL path
-        cd_model = gtk.ListStore(str, str, bool, str)
-        cd_list.set_model(cd_model)
-        text = gtk.CellRendererText()
-        cd_list.pack_start(text, True)
-        cd_list.add_attribute(text, 'text', 1)
-        cd_list.add_attribute(text, 'sensitive', 2)
-
+        self.initialize_opt_media()
         self.reset_state()
 
     def close(self,ignore1=None,ignore2=None):
@@ -81,43 +82,47 @@ class vmmChooseCD(gobject.GObject):
         win.show()
 
     def reset_state(self):
-        if self.conn.is_remote():
-            self.window.get_widget("physical-media").set_sensitive(False)
-            self.window.get_widget("iso-image").set_active(True)
-            self.window.get_widget("cd-path").set_active(-1)
-            self.window.get_widget("iso-file-chooser").set_sensitive(False)
+        cd_path = self.window.get_widget("cd-path")
+        use_cdrom = (cd_path.get_active() > -1)
+
+        if use_cdrom:
+            self.window.get_widget("physical-media").set_active(True)
         else:
-            self.window.get_widget("physical-media").set_sensitive(True)
-            self.window.get_widget("iso-file-chooser").set_sensitive(True)
-            self.populate_opt_media()
-            self.window.get_widget("cd-path").set_active(0)
+            self.window.get_widget("iso-image").set_active(True)
 
     def ok(self,ignore1=None, ignore2=None):
+        path = None
+
         if self.window.get_widget("iso-image").get_active():
             path = self.window.get_widget("iso-path").get_text()
         else:
             cd = self.window.get_widget("cd-path")
+            idx = cd.get_active()
             model = cd.get_model()
-            path = model.get_value(cd.get_active_iter(), 0)
+            if idx != -1:
+                path = model[idx][uihelpers.OPTICAL_DEV_PATH]
 
-        if path == "" or path == None: 
-            return self.err.val_err(_("Invalid Media Path"), \
+        if path == "" or path == None:
+            return self.err.val_err(_("Invalid Media Path"),
                                     _("A media path must be specified."))
 
         try:
+            dev=virtinst.VirtualDisk.DEVICE_CDROM
             disk = virtinst.VirtualDisk(path=path,
-                                        device=virtinst.VirtualDisk.DEVICE_CDROM, 
+                                        device=dev,
                                         readOnly=True,
                                         conn=self.conn.vmm)
         except Exception, e:
             return self.err.val_err(_("Invalid Media Path"), str(e))
-        self.emit("cdrom-chosen", disk.type, disk.path, self.dev_id_info)
+
+        uihelpers.check_path_search_for_qemu(self.topwin, self.config,
+                                             self.conn, path)
+
+        self.emit("cdrom-chosen", self.dev_id_info, disk.path, disk.type)
         self.cancel()
 
     def media_toggled(self, ignore1=None, ignore2=None):
         if self.window.get_widget("physical-media").get_active():
-            self.populate_opt_media()
-            self.window.get_widget("cd-path").set_active(0)
             self.window.get_widget("cd-path").set_sensitive(True)
             self.window.get_widget("iso-path").set_sensitive(False)
             self.window.get_widget("iso-file-chooser").set_sensitive(False)
@@ -130,22 +135,39 @@ class vmmChooseCD(gobject.GObject):
         pass
 
     def browse_fv_iso_location(self, ignore1=None, ignore2=None):
-        filename = self._browse_file(_("Locate ISO Image"))
-        if filename != None:
-            self.window.get_widget("iso-path").set_text(filename)
+        self._browse_file()
 
-    def populate_opt_media(self):
-        try:
-            optical_helper = vmmOpticalDriveHelper(self.window.get_widget("cd-path"))
-            optical_helper.populate_opt_media()
-            self.window.get_widget("physical-media").set_sensitive(True)
-        except Exception, e:
-            logging.error("Unable to create optical-helper widget: '%s'", e)
-            self.window.get_widget("physical-media").set_sensitive(False)
+    def initialize_opt_media(self):
+        widget = self.window.get_widget("cd-path")
+        warn = self.window.get_widget("cd-path-warn")
 
-    def _browse_file(self, dialog_name, folder=None, _type=None):
-        return util.browse_local(self.window.get_widget("vmm-choose-cd"),
-                                 dialog_name, folder, _type)
+        error = self.conn.mediadev_error
+        uihelpers.init_mediadev_combo(widget)
+        uihelpers.populate_mediadev_combo(self.conn, widget, self.media_type)
 
+        if error:
+            warn.show()
+            util.tooltip_wrapper(warn, error)
+        else:
+            warn.hide()
+
+        self.window.get_widget("physical-media").set_sensitive(not bool(error))
+
+        if self.media_type == MEDIA_FLOPPY:
+            self.window.get_widget("physical-media").set_label(
+                                                            _("Floppy D_rive"))
+            self.window.get_widget("iso-image").set_label(_("Floppy _Image"))
+
+    def set_storage_path(self, src, path):
+        self.window.get_widget("iso-path").set_text(path)
+
+    def _browse_file(self):
+        if self.storage_browser == None:
+            self.storage_browser = vmmStorageBrowser(self.config, self.conn)
+            self.storage_browser.connect("storage-browse-finish",
+                                         self.set_storage_path)
+
+        self.storage_browser.set_browse_reason(self.config.CONFIG_DIR_MEDIA)
+        self.storage_browser.show(self.conn)
 
 gobject.type_register(vmmChooseCD)
