@@ -84,14 +84,15 @@ class vmmConnect(gobject.GObject):
 
         self.window.get_widget("conn-list").get_selection().connect("changed", self.conn_selected)
 
-        self.bus = dbus.SystemBus()
+        self.bus = None
+        self.server = None
+        self.can_browse = False
         try:
+            self.bus = dbus.SystemBus()
             self.server = dbus.Interface(self.bus.get_object("org.freedesktop.Avahi", "/"), "org.freedesktop.Avahi.Server")
             self.can_browse = True
         except Exception, e:
             logging.debug("Couldn't contact avahi: %s" % str(e))
-            self.server = None
-            self.can_browse = False
 
         self.reset_state()
 
@@ -107,7 +108,6 @@ class vmmConnect(gobject.GObject):
 
     def show(self):
         win = self.window.get_widget("vmm-open-connection")
-        win.show_all()
         win.present()
         self.reset_state()
 
