@@ -22,17 +22,16 @@ import gobject
 class vmmNetDevice(gobject.GObject):
     __gsignals__ = {}
 
-    def __init__(self, config, connection, name, mac, shared, bridge=None):
+    def __init__(self, name, mac, is_shared, bridge=None, hal_path=None):
         self.__gobject_init__()
 
-        self.conn = connection
         self.name = name
         self.mac = mac
-        self.shared = shared
+        self.shared = is_shared
         self.bridge = bridge
 
-    def get_connection(self):
-        return self.conn
+        # Used for HAL backend population
+        self.hal_path = hal_path
 
     def get_name(self):
         return self.name
@@ -46,7 +45,7 @@ class vmmNetDevice(gobject.GObject):
     def get_mac(self):
         return self.mac
 
-    def get_info(self):
-        return (self.name, self.mac, self.shared, self.bridge)
+    def get_hal_path(self):
+        return self.hal_path
 
 gobject.type_register(vmmNetDevice)
